<?php
/**
 * Retrieves and creates the cg-config.php file.
 *
 * The permissions for the base directory must allow for writing files in order
 * for the cg-config.php to be created using this page.
 *
 * @package WordPress
 * @subpackage Administration
 */

/**
 * We are installing.
 *
 * @package WordPress
 */
define('CG_INSTALLING', true);

define('IN_CG', true);
/**
 * Disable error reporting
 *
 * Set this to error_reporting( E_ALL ) or error_reporting( E_ALL | E_STRICT ) f
 or debugging
 */
error_reporting(E_ALL | E_STRICT);

/**#@+
 * These three defines are required to allow us to use require_wp_db() to load
 * the database class while being cg-content/db.php aware.
 * @ignore
 */
define('ABSPATH', dirname(dirname(__FILE__)).'/'); // Absolute path for Coal Games Project since cg-load.php is not loaded
define('CG_INC', 'cg-includes/'); // Remember to add a trailing slash
define('CG_CONTENT_DIR', ABSPATH . 'cg-content');
/**#@-*/



if (!file_exists(ABSPATH . 'cg-config-sample.php'))
cg_die('Sorry, I need a cg-config-sample.php file to work from. Please re-upload this file from your Coal Games installation.');

$configFile = file(ABSPATH . 'cg-config-sample.php');

// Check if cg-config.php has been created
if (file_exists(ABSPATH . 'cg-config.php'))
die("<p>The file 'cg-config.php' already exists. If you need to reset any of the configuration items in this file, please delete it first. You may try <a href='install.php'>installing now</a>.</p>");

// Check if cg-config.php exists above the root directory but is not part of another install
//if (file_exists(ABSPATH . '../cg-config.php') && ! file_exists(ABSPATH . '../cg-settings.php'))
//    cg_die("<p>The file 'cg-config.php' already exists one level above your WordPress installation. If you need to reset any of the configuration items in this file, please delete it first. You may try <a href='install.php'>installing now</a>.</p>");

if ( version_compare( '4.3', phpversion(), '>' ) )
cg_die('Your server is running PHP version ' . phpversion() . ' but WordPress requires at least 4.3.');

if ( !extension_loaded('mysql') && !file_exists(ABSPATH . CG_INC . 'cgsql.php') )
cg_die('Your PHP installation appears to be missing the MySQL extension which is required by Coal Games.');

if ( isset($_GET['step']) )
$step = $_GET['step'];
else
$step = 0;

/**
 * Display setup cg-config.php file header.
 *
 * @ignore
 * @since 2.3.0
 * @package WordPress
 * @subpackage Installer_WP_Config
 */
function display_header() {
	header( 'Content-Type: text/html; charset=utf-8' );
	?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
<title>Coal Games Project &rsaquo; Setup Configuration File</title>
<link rel="stylesheet" href="css/install.css" type="text/css" />

</head>
<body>
<h1 id="logo">Coal Games Project</h1>
	<?php
}//end function display_header();

switch($step) {
	case 0:
		display_header();
		?>

<p>Welcome to Coal Games Project. Before getting started, we need some
information on the database. You will need to know the following items
before proceeding.</p>
<ol>
	<li>Database name</li>
	<li>Database username</li>
	<li>Database password</li>
	<li>Database host</li>
	<li>Table prefix (if you want to run more than one Coal Games in a
	single database)</li>
</ol>
<p><strong>If for any reason this automatic file creation doesn't work,
don't worry. All this does is fill in the database information to a
configuration file. You may also simply open <code>cg-config-sample.php</code>
in a text editor, fill in your information, and save it as <code>cg-config.php</code>.
</strong></p>
<p>In all likelihood, these items were supplied to you by your Web Host.
If you do not have this information, then you will need to contact them
before you can continue. If you&#8217;re all ready&hellip;</p>

<p class="step"><a href="setup-config.php?step=1" class="button">Let&#8217;s
go!</a></p>
		<?php
		break;

case 1:
	display_header();
	?>
<form method="post" action="setup-config.php?step=2">
<p>Below you should enter your database connection details. If you're
not sure about these, contact your host.</p>
<table class="form-table">
	<tr>
		<th scope="row"><label for="dbname">Database Name</label></th>
		<td><input name="dbname" id="dbname" type="text" size="25"
			value="coalgames" /></td>
		<td>The name of the database you want to run Coal Games in.</td>
	</tr>
	<tr>
		<th scope="row"><label for="uname">User Name</label></th>
		<td><input name="uname" id="uname" type="text" size="25"
			value="username" /></td>
		<td>Your MySQL username</td>
	</tr>
	<tr>
		<th scope="row"><label for="pwd">Password</label></th>
		<td><input name="pwd" id="pwd" type="text" size="25" value="password" /></td>
		<td>...and MySQL password.</td>
	</tr>
	<tr>
		<th scope="row"><label for="dbhost">Database Host</label></th>
		<td><input name="dbhost" id="dbhost" type="text" size="25"
			value="localhost" /></td>
		<td>99% chance you won't need to change this value.</td>
	</tr>
	<tr>
		<th scope="row"><label for="prefix">Table Prefix</label></th>
		<td><input name="prefix" id="prefix" type="text" id="prefix"
			value="cg_" size="25" /></td>
		<td>If you want to run multiple Coal Games installations in a single
		database, change this.</td>
	</tr>
</table>
<p class="step"><input name="submit" type="submit" value="Submit"
	class="button" /></p>
</form>
	<?php
	break;

case 2:
	$dbname  = trim($_POST['dbname']);
	$uname   = trim($_POST['uname']);
	$passwrd = trim($_POST['pwd']);
	$dbhost  = trim($_POST['dbhost']);
	$prefix  = trim($_POST['prefix']);
	if (empty($prefix)) $prefix = 'cg_';

	// Test the db connection.
	/**#@+
	* @ignore
	*/
	define('DB_NAME', $dbname);
	define('DB_USER', $uname);
	define('DB_PASSWORD', $passwrd);
	define('DB_HOST', $dbhost);
	/**#@-*/

	require_once(ABSPATH . CG_INC . 'cgsql.php');

	// We'll fail here if the values are no good.
	if (!isset($cgsql)) {
		die('$cgsql object not found.');
	}

	foreach ($configFile as $line_num => $line) {
		switch (substr($line,0,16)) {
			case "define('DB_NAME'":
				$configFile[$line_num] = str_replace("databasename", $dbname, $line);
				break;
			case "define('DB_USER'":
				$configFile[$line_num] = str_replace("'username'", "'$uname'", $line);
				break;
			case "define('DB_PASSW":
				$configFile[$line_num] = str_replace("'password'", "'$passwrd'", $line);
				break;
			case "define('DB_HOST'":
				$configFile[$line_num] = str_replace("localhost", $dbhost, $line);
				break;
			case '$table_prefix  =':
				$configFile[$line_num] = str_replace('cg_', $prefix, $line);
				break;
		}
	}
	if ( ! is_writable(ABSPATH) ) :
	display_header();
	?>
<p>Sorry, but I can't write the <code>cg-config.php</code> file.</p>
<p>You can create the <code>cg-config.php</code> manually and paste the
following text into it.</p>
<textarea cols="90" rows="15"><?php
foreach( $configFile as $line ) {
	echo htmlentities($line);
}
?></textarea>
<p>There is no need to install yet because the installer is not
finished.</p>
<p class="step"><a href="install.php" class="button">Run the install</a></p>
<?php
else :
$handle = fopen(ABSPATH . 'cg-config.php', 'w');
foreach( $configFile as $line ) {
	fwrite($handle, $line);
}
fclose($handle);
chmod(ABSPATH . 'cg-config.php', 0666);
display_header();
?>
<p>All right sparky! You've made it through this part of the
installation. Coal Games Project can now (hopefully) communicate with
your database. But, first, you have to use the dump files in the
coalgames.sql in order to have this working.<?php /*If you are ready, time now to&hellip;*/?></p>
<p>Since the installer is not yet finished, you do not have to install.</p>

<?php //<p class="step"><a href="install.php" class="button">Run the install</a></p>
endif;
break;
}
?>
</body>
</html>
