<?php
/**
 * The core files for the Coal Games Project
 *
 * Everything in WordPress is in wp-settings.php but that is not very
 * descriptive. It has been split up into two parts. cg-settings.php is
 * just for used for settings and the cg-core.php loads the functions etc.
 *
 * Used to setup and fix common variables and include
 * the Coal Games Project procedural and class library.
 */

/**
 * If file was accessed directly, exit to prevent problems.
 * First, it tries to go to cg-directfile.php. If that file
 * is not found, then it just displays a generic message.
 * This is so that you can place a link or other message on
 * a file so that if a search engine linked to it, you could
 * get a link back to your main site.
 *
 * This file SHOULD automatically edited by the installer and
 * by the administration panel but users can also edit this/
 * The installer is not yet created so the installer does not
 * use this file.
 */
if ( ! defined ( 'IN_CG' ) ) {
	if ( ! (@include ('./cg-directfile.php')) ) {
		die ( "File can not be accessed directly" );
	}
}

define ( 'CG_TIME', time () );

/**
 * Set the amount of max memory Coal Games is alllowed to
 * use. Default is at 32M but you may increase it if you
 * experience some problems.
 */
if ( function_exists ( 'memory_get_usage' ) && (( int ) @ini_get ( 'memory_limit' ) < abs ( intval ( $cg_settings['memory_limit'] ) )) )
@ini_set ( 'memory_limit', $cg_settings['memory_limit'] );

/**
 * Site_url
 */

if ($cg_settings['site_url_strict'] == false) {
	$cg_settings['site_url'] = 'http';
	if (!empty($_SERVER['HTTPS'])) {
		$cg_settings['site_url'] .= 's';
	}
	$cg_settings['site_url'] .= '://';
	if ($_SERVER['SERVER_PORT'] != '80') {
		$cg_settings['site_url'] .= $_SERVER['HTTP_HOST'] . ':' . $_SERVER['SERVER_PORT'] . $_SERVER['REQUEST_URI'];
	} else {
		$cg_settings['site_url'] .= $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'];
	}

	if (substr($cg_settings['site_url'], -1) == '/') {
		// If the last character of the current url is a forward slash, then
		// use a hack so that the correct directory is displayed correctly.
		$cg_settings['site_url'] .= '.';
	}
	$cg_settings['site_url'] = substr($cg_settings['site_url'], 0, -(strlen(substr(strrchr($cg_settings['site_url'], '/'), 0))));

}

/**
 * Quicker way to access the url to the website url but with a backslash at the end
 */
define ( 'SITE_URL', $cg_settings['site_url'] . '/' );

/**
 * Include path for files
 */
define ( 'INC_PATH', ABSPATH . $cg_settings['folder.includes'] . '/' ); // No trailing slash


/**
 * Quicker way to access the content folder
 */
define ( 'CONTENT_PATH', ABSPATH . $cg_settings['folder.content'] . '/' );

/**
 * URL to put in themes for the content folder
 */
define ( 'CONTENT_URL', SITE_URL . $cg_settings['folder.content'] . '/' );

/**
 * URL for the theme folder
 */
define ( 'THEME_URL', SITE_URL . $cg_settings['folder.content'] . '/' . $cg_settings['folder.themes'] . '/' . $cg_settings['cur_theme'] . '/' );

/**
 * PATH for the theme folder
 */
define ( 'THEME_PATH', ABSPATH . $cg_settings['folder.content'] . '/' . $cg_settings['folder.themes'] . '/' . $cg_settings['cur_theme'] . '/' );

/**
 * PATH for the flash games
 */
define ( 'GAMES_PATH', ABSPATH . $cg_settings['folder.content'] . '/' . $cg_settings['folder.flashgames'] . '/' );

/**
 * URL for the flash games
 */
define ( 'GAMES_URL', SITE_URL . $cg_settings['folder.content'] . '/' . $cg_settings['folder.flashgames'] . '/' );

/**
 * Game Icon url
 */
define ( 'GAMEICON_URL', SITE_URL . $cg_settings['folder.content'] . '/' . $cg_settings['folder.gameicons'] . '/' );

/**
 * The server's path for the admin folder
 */
define ( 'ADMIN_PATH', ABSPATH . $cg_settings['folder.admin'] . '/' );

/**
 * URL for the admin folder
 */
define ( 'ADMIN_URL', SITE_URL . $cg_settings['folder.admin'] . '/' );

/**
 * Cache path
 */
define ( 'CACHE_PATH', INC_PATH . 'cache/' );

/*
 // Add define('CG_DEBUG', true); to cg-config.php to enable display of notices during development.
 if ( defined('CG_DEBUG') && CG_DEBUG ) {
 if ( defined('E_DEPRECATED') )
 error_reporting(E_ALL & ~E_DEPRECATED & ~E_STRICT);
 else
 error_reporting(E_ALL);
 // Add define('CG_DEBUG_DISPLAY', false); to wp-config.php to use the globally configured setting for display_errors and not force it to On
 if ( ! defined('CG_DEBUG_DISPLAY') || CG_DEBUG_DISPLAY ) {
 ini_set('display_errors', 1);
 }
 // Add define('CG_DEBUG_LOG', true); to enable php debug logging to CG_CONTENT_DIR/debug.log
 if ( defined('CG_DEBUG_LOG') && CG_DEBUG_LOG ) {
 ini_set('log_errors', 1);
 $admin_path = ADMIN_PATH;
 ini_set('error_log', $admin_path . 'debug.log');
 }
 } else {
 define('CG_DEBUG', false);
 if ( defined('E_RECOVERABLE_ERROR') ) {
 error_reporting(E_ERROR | E_WARNING | E_PARSE | E_USER_ERROR | E_USER_WARNING | E_RECOVERABLE_ERROR);
 }
 else {
 error_reporting(E_ERROR | E_WARNING | E_PARSE | E_USER_ERROR | E_USER_WARNING);
 }
 }*/
/* Some PHP 5.3 stuff
 if (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION != 3) {

 }*/
/**
 * Check for magic quotes
 */
if ( function_exists ( 'get_magic_quotes_gpc' ) && get_magic_quotes_gpc () ) {
	/**
	 * Trim the input of and slashes created by magic quotes and removing
	 * whitespace at the beginning and end of each value in the array.
	 */
	$cg_post = array_map ( 'stripslashes', array_map ( 'trim', $_POST ) );
	$cg_get = array_map ( 'stripslashes', array_map ( 'trim', $_GET ) );
	$cg_cookie = array_map ( 'stripslashes', array_map ( 'trim', $_COOKIE ) );
} /**
* If magic quotes does not exist or is not enabled, the string
* will only be trimmed of extra whitespace.
*/
else {
	/**
	 * Since magic quotes is not enabled, the array only needs to
	 * be trimmed from the front and the back.
	 */
	$cg_post = array_map ( 'trim', $_POST ); // Trim the input of any excess material
	$cg_get = array_map ( 'trim', $_GET );
	$cg_cookie = array_map ( 'trim', $_COOKIE ); // Trim the input of any excess material
}

/**
 * Turn register globals off.
 *
 * @access private
 * @since WordPress 2.1.0
 * @return null Will return null if register_globals PHP directive was disabled
 */
function cg_unregister_GLOBALS() {
	if ( ! ini_get ( 'register_globals' ) )
	return;

	if ( isset ( $_REQUEST['GLOBALS'] ) )
	die ( 'GLOBALS overwrite attempt detected' );

	// Variables that shouldn't be unset
	$noUnset = array ('GLOBALS', '_GET', '_POST', '_COOKIE', '_REQUEST', '_SERVER', '_ENV', '_FILES', 'table_prefix' );

	$input = array_merge ( $_GET, $_POST, $_COOKIE, $_SERVER, $_ENV, $_FILES, isset ( $_SESSION ) && is_array ( $_SESSION ) ? $_SESSION : array () );
	foreach ( $input as $k => $v )
	if ( ! in_array ( $k, $noUnset ) && isset ( $GLOBALS[$k] ) ) {
		$GLOBALS[$k] = NULL;
		unset ( $GLOBALS[$k] );
	}
}

cg_unregister_GLOBALS ();



require INC_PATH . 'formatting.php';

/**
 * Initiate the CgSql
 */
if ( ! defined ( 'NO_CGSQL' ) ) {
	require INC_PATH . 'cgsql.php';
}

require INC_PATH . 'sessions.php';

//require INC_PATH . 'users.php';

$cg_categories = $cgsql->get_results('SELECT * FROM `' . CATEGORIES_TABLE . '`');

/**
 * This variable is so that we should no what to processe for the template
 *
 * @var int
 */
$cg_query = 0; // Default



require INC_PATH . 'cg_template.php';