<?php
/**
 * User sessions file.
 * 
 * This file will check the user against the database for any record matching.
 * This file is meant to be included.
 * This file is also assuming that the user already has a cookie and
 * this page needs the appropriate variables/constants to run properly:
 * SESSION_COOKIE
 * $cgsql abstraction layer is avaliable (connected or not)
 *
 * Copyright Joshua Li 2010
 *
 * @package Coalgames
 * @subpackage sessions
 * @since 4-28-2009
 * @version 5-28-2009
 */

/**
 * @ignore Prevents malicious users
 */
if ( ! defined ( 'IN_CG' ) ) {
	exit ();
}
/**
 * Variable that is avaliable for the templates
 */
$cg_session = array ('status' => false, 'username' => '' );

/**
 * Variable where all the processing is done to reduce many names.
 */
$cg_sessionData = array ();

/**
 * The session functions. We might need this if we are in tasks.
 */
require INC_PATH . 'session-functions.php';

if ( isset ( $cg_cookie[$cg_settings['cookie.session']] ) && strlen ( $cg_cookie[$cg_settings['cookie.session']] ) === 40 && preg_match ( '/^[a-f0-9]{40}$/i', $cg_cookie[$cg_settings['cookie.session']] ) && !defined('NO_SESSION') ) {
	echo '<pre>';
	print_r($cg_cookie);
	echo '</pre>';

	/**
	 * Generate an ipagent hash before going into the sql connection.
	 */
	$cg_agent_hash = sha1($_SERVER['HTTP_USER_AGENT']);

	$cg_session_query = 'SELECT * FROM ' . SESSIONS_TABLE . ' WHERE `session_hash`=\'' . $cg_cookie[$cg_settings['cookie.session']] . '\' && `agent_hash` = \'' . $cg_agent_hash . '\' LIMIT 1';
	$cg_session_data['session_result'] = $cgsql->simple_query ( $cg_session_query );
	if ( 1 == $cgsql->num_rows ( $cg_session_data['session_result'] ) ) {
		
		$cg_session['status'] = true;


		$cg_session_data['session_data'] = $cgsql->fetch($cg_session_data['session_result']);
		$cg_session['user_id'] = $cg_session_data['session_data']['user_id'];
		$cg_session_data['user_query'] = 'SELECT * FROM `' . USERS_TABLE . '` WHERE `user_id`=\'' . $cg_session_data['session_data']['user_id'] . '\'';
		$cg_session_data['user_results'] = $cgsql->get_row($cg_session_data['user_query']);
		/**
		 * Sometimes, the user gets deleted but the sessions dont get deleted..
		 * This check makes sure that if this happens, there will be no errors.
		 * Also, the session will be deleted and the cookie will also be deleted.
		 */
		if ($cg_session_data['user_results'] != false) {
			$cg_session_data['userdata_query'] = 'SELECT * FROM `' . USERDATA_TABLE . '` WHERE `user_id`=\'' . $cg_session_data['session_data']['user_id'] . '\'';
			$cg_session_data['userdata_results'] = $cgsql->get_row($cg_session_data['userdata_query']);
			$cg_session = array_merge ( $cg_session_data['session_data'], $cg_session, $cg_session_data['userdata_results'], $cg_session_data['user_results'] );
			
			/**
			 * Get info about the usergroup that the is in.
			 */
			echo $cg_session_data['usergroup_query'] = 'SELECT * FROM `' . USERGROUPS_TABLE . '` WHERE `usergroup_id`=' . $cg_session['usergroup_id'];
			
			/**
			 * Fetch all the information about that usergroup
			 */
			$cg_session_data['usergroup_results'] = $cgsql->get_results($cg_session_data['usergroup_query']);
			
			/**
			 * Add the newly retrieved usergroup data to our $cg_session array.
			 */
			$cg_session = array_merge ( $cg_session_data['usergroup_results'][0], $cg_session);
			
			/**
			 * See if we can do some today_on logic
			 */
			if ( $cg_settings['on.today_on'] && $cg_session['today_on'] == 0 ) {
				/**
				 * Add the points to the user for the today_on.
				 */
				$cg_today_on_query = 'UPDATE `' . USERDATA_TABLE . '` SET `today_on`=1, `points`=`points`+' . $cg_settings['points.today_on'] . ', `lifetime_points`=`lifetime_points`+' . $cg_settings['points.today_on'] . ' WHERE `user_id`=' . $cg_session['user_id'] . ' LIMIT 1';
				$cgsql->simple_query($cg_today_on_query);

				if ( $cgsql->affected_rows() == 1 ) {
					$cg_session['points'] += $cg_settings['points.today_on'];
				}
			}

			$cg_max_active = $cg_session['last_active'] + $cg_settings['user.active_interval'];
			$cg_session['recent_active'] = CG_TIME - $cg_session['last_active'];
			if ( $cg_settings['on.total_active'] === true && $cg_session['recent_active'] > $cg_settings['user.time_interval'] ) {


				if ( $cg_max_active > CG_TIME && $cg_session['recent_active']) {
					$cg_session['total_active'] = $cg_session['total_active'] + $cg_session['recent_active'];
					$cg_update_active_query = 'UPDATE `' . USERDATA_TABLE . '` SET `last_active`=\'' . CG_TIME . '\', `total_active`=`total_active`+' . $cg_session['recent_active'] . ' WHERE `user_id`=\'' . $cg_session['user_id'] . '\' LIMIT 1';
					/*if ( $cgsql->affected_rows ( $cgsql->simple_query ( $cg_update_active_query ) ) != 1 ) {
						echo 'Error Code # 5';
					}
					No need to check if we had an error
					*/
				} else {
						$cg_last_active_query = 'UPDATE `' . USERDATA_TABLE . '` SET `last_active`=\'' . CG_TIME . '\' WHERE `user_id`=\'' . $cg_session['user_id'] . '\' LIMIT 1';
						$cgsql->simple_query($cg_last_active_query);
				}
			}
			
			/**
			 * Refresh the user's session if they are running out of time soon.
			 * This will make it so that if the user is still on the site, they will not be suddenly disconnected.
			 */
			$cg_session['refresh.time'] = $cg_session['expire'] - $cg_settings['session.refresh'];
			$cg_session['refresh.min_time'] = ($cg_session['expire'] + $cg_settings['session.refresh']) + $cg_settings['session.min_refresh'];
			/**
			 * Check if we can extend the user's session time
			 */
			if ($cg_session['refresh.time'] < CG_TIME && CG_TIME < $cg_session['refresh.min_time']) {
				$cgsql->simple_query('UPDATE `' . SESSIONS_TABLE . '` SET `expire`=' . ($cg_session['expire'] + $cg_settings['session.refresh']) . ' WHERE `session_hash`=\'' . $cg_session['session_hash'] . '\'');
				setcookie($cg_settings['cookie.session'], $cg_session['session_hash'], ($cg_session['expire'] + $cg_settings['session.refresh']), '/', '', 0, 1);
			}
		} else { 
			/**
			 * If there is a session found but the user does not exist because it may have been deleted, then we should
			 * attempt delete the cookie from the user and delete the session in the database.
			 */
			echo 'The user could not be found!';
			setcookie($cg_settings['cookie.session'], 0, 0, '/', '', 0, 1);
			echo $cg_session_delete_query = 'DELETE FROM `' . SESSIONS_TABLE . '` WHERE `session_hash`=\'' . $cg_cookie[$cg_settings['cookie.session']] . '\' && `agent_hash`=\'' . $cg_agent_hash . '\' LIMIT 1';
			$cgsql->simple_query($cg_session_delete_query); // Delete the session
			$cg_session['status'] = false;
		}


		/*
		$cg_session_data['session_data'] = $cgsql->fetch($cg_session_data['session_result']);
		$cg_session['user_id'] = $cgsql->fetch($cg_session_data['session_result']);
		$cg_session['user_query'] = 'SELECT `' . USERS_TABLE . '`.`username`, `' . USERDATA_TABLE . '`.`rank`, `' . USERDATA_TABLE . '`.`last_active`, `' . USERDATA_TABLE . '`.`total_active`, `' . USERDATA_TABLE . '`.`points`, `' . USERDATA_TABLE . '`.`today_on` FROM `' . USERS_TABLE . '`, `' . USERDATA_TABLE . '` WHERE `' . USERDATA_TABLE . '`.`user_id`=`' . USERS_TABLE . '`.`user_id` AND `' . USERDATA_TABLE . '`.`user_id`=\'' . $cg_session['user_id'] . '\' LIMIT 1';
		$cg_sessionData['user_data'] = $cgsql->fetch ( $cgsql->query ( $cg_session['user_query'] ) );
		if ( $cg_sessionData['user_data']['today_on'] == 0 && $GLOBALS['cg_settings']['on.today_on'] ) {
			if ( $cgsql->affected_rows ( $cgsql->query ( 'UPDATE ' . USERDATA_TABLE . ' SET `today_on`=1, `points`=' . ($cg_sessionData['user_data']['points'] + $GLOBALS['cg_settings']['user.today_on_reward']) . ' WHERE `user_id`=' . $cg_session['user_id'] . ' LIMIT 1' ) ) === 1 ) {
				$cg_sessionData['user_data']['points'] = $cg_sessionData['user_data']['points'] + $GLOBALS['cg_settings']['user.today_on_reward'];
			}
		}
		if ( $GLOBALS['cg_settings']['on.active_interval'] === true ) {
			$cg_sessionData['active_time'] = $cg_sessionData['user_data']['last_active'] + $GLOBALS['cg_settings']['user.active_interval'];
			if ( $cg_sessionData['active_time'] > time () ) {
				$cg_sessionData['recent_active'] = time () - $cg_sessionData['user_data']['last_active'];
				$cg_session['time_active'] = $cg_sessionData['user_data']['total_active'] + $cg_sessionData['recent_active'];
				if ( $cgsql->affected_rows ( $cgsql->query ( 'UPDATE `' . USERDATA_TABLE . '` SET `last_active`=\'' . time () . '\', `total_active`=\'' . $cg_session['time_active'] . '\' WHERE `user_id`=\'' . $cg_session['user_id'] . '\' LIMIT 1' ) ) != 1 ) {
					//echo $GLOBALS['cg_settings']['error.pfx'] . 'Error Code # 5';
				}
			} else {
				$cg_session['time_active'] = $cg_sessionData['user_data']['total_active'];
				if ( $cgsql->affected_rows ( $cgsql->query ( 'UPDATE `' . USERDATA_TABLE . '` SET `last_active`=\'' . time () . '\' WHERE `user_id`=\'' . $cg_session['user_id'] . '\' LIMIT 1' ) ) != 1 ) {
					echo $GLOBALS['cg_settings']['error.pfx'] . 'Error Code # 6';
				}
			}
		}
		$cg_session = array_merge ( $cg_session, $cg_sessionData['user_data'] );
		$cg_session['points'] = number_format ( $cg_session['points'] );*/
		
		echo '<pre>';print_r($cg_session);echo '</pre>';

		/**
		 * Just for a test, lets add 1 point to the user
		 */
		change_points(1);
	} else {
		// The user does not have a valid cookie ... do nothing
	} // End of if: 1 row returned
}